#!/bin/bash

function request_with_nic(){
    local nic=$1
    # 获取ip方式
    # https://api.myip.com
    # https://ipinfo.io
    # https://cip.cc
    # https://myip.ipip.net
    # https://ifconfig.me
    # http://members.3322.org/dyndns/getip

    local result=
    local ip=
    result=$(curl --interface ${nic}  -X GET https://ipinfo.io)
    ip=$(echo $result | jq .ip | tr -d '"')
    # ip="192.168.2.153"
    # 判断是否返回ip
    if [ -z "$ip" ]; then
        # 此api可能请求到ipv6，导致不匹配
        result=$(curl --interface ${nic}  -X GET https://api.myip.com)
        ip=$(echo $result | jq .ip | tr -d '"')
    fi
    if [ -z "$ip" ]; then
        ip=$(curl --interface ${nic}  -X GET https://ifconfig.me)
    fi

    # if [ "curl -sL --interface eth1 -w "%{http_code}" "https://ifconfig.me" -o /dev/null" = "200" ]; then
    # 判断 ip 是否相等
    # echo "ifconfig $(ifconfig ${nic} | grep inet | grep -v inet6 | awk '{print $2}')"
    # echo "ip ${ip}"

    if [ -n "$ip" ]; then 
    # if [[ "$ip" == "$(ifconfig ${nic} | grep inet | grep -v inet6 | awk '{print $2}')" ]]; then 
        echo 1
        return 1
    fi
    echo 0
    return 0
}

RECORD_LOG="/root/pear/log/network.log"
mkdir -p /root/pear/var/

function detect_nic(){
    local info=$1
    local nic=`echo $info | jq .interface | tr -d '"'`
    local ip=`echo $info | jq .ipaddr | tr -d '"'`
    local gw=`echo $info | jq .gateway | tr -d '"'`
    local mask=`echo $info | jq .mask | tr -d '"'`
    
    local count=$(cat /root/pear/var/$nic)
    local checkIPRule=$(ip ro ls tab ${mask})
    if [ -z "${checkIPRule}" ] && [[ ${checkIPRule} != "default via *" ]];then
        echo `date "+%Y-%m-%d %H:%M:%S"`" 路由掉线 ${nic} ${mask} err: 100" >> ${RECORD_LOG}
        /etc/router/scripts/route.sh -s "$nic $ip $gw $mask"
    fi

    local result=`request_with_nic $nic`
    if [ $result -eq 0 ]; then 
        echo `date "+%Y-%m-%d %H:%M:%S"`" 网卡请求失败 ${nic}" >> ${RECORD_LOG}
        count=$(($count+1))
        echo $count > /root/pear/var/$nic
    else
        echo `date "+%Y-%m-%d %H:%M:%S"`" 网卡状态正常 ${nic}" >> ${RECORD_LOG}
        echo 0 > /root/pear/var/$nic
        return; # 已恢复
    fi

    if [ $count -eq 5 ]; then 
        
        echo `date "+%Y-%m-%d %H:%M:%S"`" 强制重刷路由 ${nic} ${mask}" >> ${RECORD_LOG}
        /etc/router/scripts/route.sh -s "$nic $ip $gw $mask"

        result=`request_with_nic $nic`
        if [ $result -eq 1 ]; then 
            echo `date "+%Y-%m-%d %H:%M:%S"`" 强刷路由后可用 ${nic} err: 100" >> ${RECORD_LOG}
            echo 0 > /root/pear/var/$nic
            return;
        else
            echo `date "+%Y-%m-%d %H:%M:%S"`" 强刷路由后不可用 ${nic}" >> ${RECORD_LOG}
        fi
        
        echo `date "+%Y-%m-%d %H:%M:%S"`" 尝试重启网卡 ${nic}" >> ${RECORD_LOG}
        ifconfig ${nic} down
        sleep 3
        ifconfig ${nic} up
        /etc/router/scripts/route.sh -s "$nic $ip $gw $mask"

        result=`request_with_nic $nic`
        if [ $result -eq 1 ]; then 
            echo `date "+%Y-%m-%d %H:%M:%S"`" 网卡假死！已恢复 ${nic} err: 102" >> ${RECORD_LOG}
        else
            echo `date "+%Y-%m-%d %H:%M:%S"`" 网络重连失败 ${nic} err: 103" >> ${RECORD_LOG}
        fi
        echo 0 > /root/pear/var/$nic
    fi
}

if [[  "$(jq .setup /etc/router/config/wan_method.json)" != "8"  ]]; then
    exit 0;
fi

MONITOR_IP_FILE=/etc/router/config/monitor_multi_static_ip.json
jq -c '.[]' ${MONITOR_IP_FILE} | while read info; do
    detect_nic $info &
done