#!/bin/bash

cfgfile_2g=/etc/router/config/wireless_2g.json
default_channel=0

start_wireless_2g() {
	local ifname=$1
	hostapd_cfgpath=/etc/router/run/hostapd.conf-2.4g-${ifname}
	enabled=`cat ${cfgfile_2g} | jq -r .enabled`
	ssid=`cat ${cfgfile_2g} | jq -r .ssid`
	wpa=`cat ${cfgfile_2g} | jq -r .encryption`
	key=`cat ${cfgfile_2g} | jq -r .key`

	if [[ ${enabled} == 0 ]]; then
		last_proc=`ps -ef | grep ${hostapd_cfgpath} | grep -v grep | awk '{print $2}'`
		if [[ -n ${last_proc} ]]; then
			hostapd_cli -i ${ifname} disable
			sleep 1
			kill -9 ${last_proc}
		fi
		echo "${ifname} disabled success!"
		return 0
	fi

	rm_driver
	sleep 1
	install_driver

	country=`cat ${cfgfile_2g} | jq -r .country`
	if [[ ${country} == null ]]; then
		country=US
	fi
	txpower=`cat ${cfgfile_2g} | jq -r .txpower`
	if [[ ${txpower} == null ]]; then
		txpower=20
	fi
	channel=`cat ${cfgfile_2g} | jq -r .channel`
	if [[ ${channel} == null || ${setup} == 7 ]]; then
		channel=${default_channel}
	fi
	hide_ssid=`cat ${cfgfile_2g} | jq -r .hide`
	if [[ ${hide_ssid} == null ]]; then
		hide_ssid=0
	fi

	# reload hostapd config
	iwconfig ${ifname} txpower ${txpower}
	# 由于hos
	ap_status=`hostapd_cli -i ${ifname} status | grep state | awk -F= '{print $2}'`
	if [[ ${ap_status} == "ENABLED" ]]; then
		hostapd_cli -i ${ifname} set country_code ${country}
		hostapd_cli -i ${ifname} set channel ${channel}
		hostapd_cli -i ${ifname} set ignore_broadcast_ssid ${hide_ssid}
		hostapd_cli -i ${ifname} set ssid ${ssid}
		hostapd_cli -i ${ifname} set wpa ${wpa}
		if [[ ${wpa} != 0 ]]; then
			hostapd_cli -i ${ifname} set wpa_passphrase ${key}
		fi

		hostapd_cli -i ${ifname} disable
		sleep 1 
		hostapd_cli -i ${ifname} enable
		# hostapd_cli -i ${ifname} reload
		ap_status=`hostapd_cli -i ${ifname} status | grep state | awk -F= '{print $2}'`
		if [[ ${ap_status} == "ENABLED" ]]; then
			echo "${ifname} reload success!"
			return 0
		else
			echo "${ifname} reload failed!" >&2
		fi
	fi

	# restart hostapd process
	last_proc=`ps -ef | grep ${hostapd_cfgpath} | grep -v grep | awk '{print $2}'`
	if [[ -n ${last_proc} ]]; then
		hostapd_cli -i ${ifname} disable
		sleep 1 
		kill -9 ${last_proc}
	fi

	cat > ${hostapd_cfgpath} << EOF
ctrl_interface=/var/run/hostapd
driver=nl80211
hw_mode=g
wmm_enabled=1
bridge=br-lan
acs_num_scans=1
ieee80211n=1
ht_capab=[HT40+][SHORT-GI-20][SHORT-GI-40]
channel=${channel}
country_code=${country}
interface=${ifname}
ignore_broadcast_ssid=${hide_ssid}
ssid=${ssid}
wpa=${wpa}
EOF
	
	if [[ ${wpa} != 0 ]]; then
		cat >> ${hostapd_cfgpath} << EOF
wpa_passphrase=${key}
auth_algs=1
wpa_key_mgmt=WPA-PSK
wpa_pairwise=TKIP CCMP
rsn_pairwise=CCMP
EOF
	fi

	hostapd -B ${hostapd_cfgpath}
	for i in {1..6}
	do
		proc=`ps -ef | grep ${hostapd_cfgpath} | grep -v grep | awk '{print $2}'`
		if [[ -z ${proc} ]]; then
			systemctl restart wpa_supplicant
			hostapd -B ${hostapd_cfgpath}
		fi
		sleep 1
		state=`hostapd_cli -i ${ifname} status 2>/dev/null | grep state | awk -F= '{print $2}'`
		if [[ ${state} == "ENABLED" ]]; then
			echo "${ifname} start success!"
			return 0
		fi
	done
	echo "${ifname} start failed!" >&2
	return 1
}

check_wireless_channel() {
	local netdev=$1
	wireless_dir="/sys/class/net/"${netdev}"/wireless"
	if [[ -d ${wireless_dir} ]]; then
		#phy_name=`cat /sys/class/net/${netdev}/phy80211/name`
		# support_5g=`iw ${phy_name} channels | grep "Band 2:"`
		# if [[ -z ${support_5g} ]]; then
		# 	start_wireless_2g ${netdev}
		# fi
		support_5g=`iwlist ${netdev} channel | grep 'Channel 36'`
		if [[ -z ${support_5g} ]]; then
			start_wireless_2g ${netdev}
		fi
	fi
}

scan_wireless_dev() {
	netdevs=`ls /sys/class/net/ | grep -v "$(ls /sys/devices/virtual/net/)"`
	for netdev in ${netdevs[@]}
	do
		check_wireless_channel ${netdev}
	done
}

rm_driver(){
	rmmod rtl8821ae # 根据网卡此条可能不存在
	rmmod rtl_pci # 根据网卡此条可能不存在
	rmmod btcoexist
	rmmod rtlwifi # 根据网卡此条可能不存在
	rmmod mac80211
	rmmod cfg80211
}

install_driver(){
	modprobe cfg80211
	modprobe lib80211
	modprobe mac80211
	modprobe rtl_pci # 根据网卡此条可能不存在
	modprobe rtlwifi # 根据网卡此条可能不存在
	modprobe rtl8821ae # 根据网卡此条可能不存在
	modprobe libarc4
	modprobe btcoexist

}

cfgfile_wan_method=/etc/router/config/wan_method.json
setup=`cat ${cfgfile_wan_method} | jq -r .setup`

if [[ ${setup} == 7 ]]; then
	cfgfile_wan=/etc/router/config/wan_wireless_repeater.json
	default_channel=`cat ${cfgfile_wan} | jq -r .channel`
	if [[ ${default_channel} -gt 14 ]]; then
		exit 0
	fi
	netdev=wlan0-ap
	start_wireless_2g ${netdev}
else
	scan_wireless_dev
fi
