#!/bin/bash

cfgfile_5g=/etc/router/config/wireless_5g.json
default_channel=0

stop_wireless(){
	local ifname=$1
	local hostapd_cfgpath=$2
	
	local last_proc=`ps -ef | grep ${hostapd_cfgpath} | grep -v grep | awk '{print $2}'`
	if [[ -n ${last_proc} ]]; then
		hostapd_cli -i ${ifname} disable
		sleep 1
		kill -9 ${last_proc}
		# reinstall_driver $ifname
	fi
	echo "${ifname} disabled success!"
	return 0
}
start_wireless_5g() {
	local ifname=$1
	local hostapd_cfgpath=/etc/router/run/hostapd.conf-5g-${ifname}
	enabled=`cat ${cfgfile_5g} | jq -r .enabled`
	ssid=`cat ${cfgfile_5g} | jq -r .ssid`
	wpa=`cat ${cfgfile_5g} | jq -r .encryption`
	key=`cat ${cfgfile_5g} | jq -r .key`

	nmcli device set ${ifname} managed no

	if [[ ${enabled} == 0 ]]; then
		stop_wireless $ifname $hostapd_cfgpath
		echo "${ifname} disabled success!"
		return 0
	fi


	hide_ssid=`cat ${cfgfile_5g} | jq -r .hide`
	if [[ ${hide_ssid} == null ]]; then
		hide_ssid=0
	fi
	country=`cat ${cfgfile_5g} | jq -r .country`
	if [[ ${country} == null ]]; then
		country=US
	fi
	txpower=`cat ${cfgfile_5g} | jq -r .txpower`
	if [[ ${txpower} == null ]]; then
		txpower=20
	fi
	chwidth=`cat ${cfgfile_5g} | jq -r .chwidth`
	if [[ ${chwidth} == null ]]; then
		chwidth=1
	fi
	channel=`cat ${cfgfile_5g} | jq -r .channel`
	if [[ ${channel} == null || ${setup} == 7 ]]; then
		channel=${default_channel}
	fi

	# wait driver start 
	while [ ! -f "/sys/class/net/${ifname}/phy80211/name" ]; do
		echo "wait phy"
		sleep 1
	done
	phy_name=`cat /sys/class/net/${ifname}/phy80211/name`
	echo $phy_name

	# wait driver start 
	# sleep 3

	same_country=`iw ${phy_name} reg get | grep "country ${country}"`
	if [[ -z ${same_country} ]]; then
		iw reg set ${country}
		sleep 1
	fi

	#valid_channel=`iwlist ${ifname} channel | grep "Channel ${channel}"`
	valid_channel=`iw ${phy_name} channels | grep -v disabled | grep " MHz \[${channel}\]"`
	if [[ -z ${valid_channel} ]]; then
		channel=0
	fi

	# reload hostapd config
	iwconfig ${ifname} txpower ${txpower}


	# ap_status=`hostapd_cli -i ${ifname} status | grep state | awk -F= '{print $2}'`
	# if [[ ${ap_status} == "ENABLED" ]]; then
	# 	hostapd_cli -i ${ifname} set country_code ${country}
	# 	hostapd_cli -i ${ifname} set vht_oper_chwidth ${chwidth}
	# 	hostapd_cli -i ${ifname} set channel ${channel}
	# 	hostapd_cli -i ${ifname} set ignore_broadcast_ssid ${hide_ssid}
	# 	hostapd_cli -i ${ifname} set ssid ${ssid}
	# 	hostapd_cli -i ${ifname} set wpa ${wpa}
	# 	if [[ ${wpa} != 0 ]]; then
	# 		hostapd_cli -i ${ifname} set wpa_passphrase ${key}
	# 	fi

	# 	# same_country=`iw reg get | grep "country ${country}"`
	# 	# if [[ -z ${same_country} ]]; then
	# 	# 	hostapd_cli -i ${ifname} disable
	# 	# 	hostapd_cli -i ${ifname} enable
	# 	# else
	# 	# 	hostapd_cli -i ${ifname} reload
	# 	# fi
		
	# 	hostapd_cli -i ${ifname} disable
	# 	sleep 1 
	# 	hostapd_cli -i ${ifname} enable
	# 	ap_status=`hostapd_cli -i ${ifname} status | grep state | awk -F= '{print $2}'`
	# 	if [[ ${ap_status} == "ENABLED" ]]; then
	# 		echo "${ifname} reload success!"
	# 		return 0
	# 	else
	# 		echo "${ifname} reload failed!" >&2
	# 	fi
	# fi
	
	start=`date +"%s"`
	echo `date +"%F %T"`"------------start-----------" > /etc/router/run/wifi.log

	# restart hostapd process
	last_proc=`ps -ef | grep ${hostapd_cfgpath} | grep -v grep | awk '{print $2}'`
	if [[ -n ${last_proc} ]]; then
		hostapd_cli -i ${ifname} disable
		sleep 1 
		kill -9 ${last_proc}
	fi

	freq_seg0_idx=0
	if [[ ${channel} -ge 36 && ${channel} -le 48 ]]; then
		freq_seg0_idx=42
	elif [[ ${channel} -ge 52 && ${channel} -le 64 ]]; then
		freq_seg0_idx=58
	elif [[ ${channel} -ge 100 && ${channel} -le 112 ]]; then
		freq_seg0_idx=106
	elif [[ ${channel} -ge 116 && ${channel} -le 128 ]]; then
		freq_seg0_idx=122
	elif [[ ${channel} -ge 132 && ${channel} -le 144 ]]; then
		freq_seg0_idx=138
	elif [[ ${channel} -ge 149 && ${channel} -le 161 ]]; then
		freq_seg0_idx=155
	elif [[ ${channel} -ge 165 && ${channel} -le 173 ]]; then
		freq_seg0_idx=167
	fi

	cat > ${hostapd_cfgpath} << EOF
ctrl_interface=/var/run/hostapd
driver=nl80211
hw_mode=a
wmm_enabled=1
bridge=br-lan
acs_num_scans=1

ieee80211n=1
ieee80211ac=1
ieee80211d=1
# ieee80211h=1

vht_oper_chwidth=${chwidth}
vht_oper_centr_freq_seg0_idx=${freq_seg0_idx}
channel=${channel}
country_code=${country}

interface=${ifname}
ignore_broadcast_ssid=${hide_ssid}
ssid=${ssid}
eap_server=0

EOF
	if [[ ${wpa} == 0 ]]; then
		cat >> ${hostapd_cfgpath} << EOF
wpa=0
EOF
	elif [[ ${wpa} == 1 ]] || [[ ${wpa} == 2 ]] || [[ ${wpa} == 3 ]]; then
		cat >> ${hostapd_cfgpath} << EOF
wpa=2
wpa_passphrase=${key}
auth_algs=1
wpa_key_mgmt=WPA-PSK
wpa_pairwise=TKIP CCMP
rsn_pairwise=CCMP
EOF
	elif [[ ${wpa}==4 ]]; then
	cat >> ${hostapd_cfgpath} << EOF
ieee80211w=2
wpa=2
auth_algs=1
wpa_key_mgmt=WPA-PSK SAE
sae_pwe=2
wpa_pairwise=TKIP CCMP
rsn_pairwise=CCMP
wpa_passphrase=${key}
nas_identifier=13
EOF
	fi

	#netcard_model=`ethtool -i ${ifname} | grep driver | awk '{print $2}'`
	bus_id=`ls -l /sys/class/net/${ifname}/device | awk -F'0000:' '{print $2}'`
	netcard_model=`lspci -s ${bus_id} | awk -F': ' '{print $2}'`
	if [[ ${netcard_model} =~ "QCA9984" ]]; then
		cat >> ${hostapd_cfgpath} << EOF
ht_capab=[LDPC][HT40+][SHORT-GI-20][SHORT-GI-40][TX-STBC][RX-STBC1][MAX-AMSDU-7935][DSSS_CCK-40]
vht_capab=[MAX-MPDU-11454][VHT160-80PLUS80][RXLDPC][SHORT-GI-80][SHORT-GI-160][TX-STBC-2BY1][RX-STBC-1][SU-BEAMFORMER][SU-BEAMFORMEE][MU-BEAMFORMEE][RX-ANTENNA-PATTERN][TX-ANTENNA-PATTERN][MAX-A-MPDU-LEN-EXP7]
EOF
	elif [[ ${netcard_model} =~ "7615" ]]; then
		cat >> ${hostapd_cfgpath} << EOF
ht_capab=[LDPC][HT40+][SHORT-GI-20][SHORT-GI-40][TX-STBC][RX-STBC1][DSSS_CCK-40]
vht_capab=[MAX-MPDU-3895][VHT160-80PLUS80][RXLDPC][SHORT-GI-80][SHORT-GI-160][TX-STBC-2BY1][RX-STBC-1][RX-ANTENNA-PATTERN][TX-ANTENNA-PATTERN][MAX-A-MPDU-LEN-EXP7]
EOF
	elif [[ ${netcard_model} =~ "Marvell Technology" ]]; then
		cat >> ${hostapd_cfgpath} << EOF
ht_capab=[LDPC][HT40+][SHORT-GI-20][SHORT-GI-40][TX-STBC][RX-STBC3]
vht_capab=[MAX-MPDU-3895][RXLDPC][SHORT-GI-80][TX-STBC-2BY1][RX-STBC-3][RX-ANTENNA-PATTERN][TX-ANTENNA-PATTERN][MAX-A-MPDU-LEN-EXP7]
EOF
	elif [[ ${netcard_model} =~ "MT7915E" ]]; then
		cat >> ${hostapd_cfgpath} << EOF
ht_capab=[RXLDPC][HT40+][SHORT-GI-20][SHORT-GI-40][TX-STBC][RX-STBC1][MAX-AMSDU-LENGTH-7935][DSSS_CCK-40]
vht_capab=[RXLDPC][TXSTBC][SHORT-GI-80][MAX-MPDU-LENGTH7991][SU-BEAMFORMER][SU-BEAMFORMEE][MU-BEAMFORMER][MU-BEAMFORMEE] [TX-ANTENNA-PATTERN][RX-ANTENNA-PATTERN][TX-1024-QAM][RX-1024-QAM][RX-STBC-1][SHOTRT-GI-160][HE160_5GHz][HE160_HE80PLUS80_5GHz][HE40_HE80_5GHz]

ieee80211ax=1
he_oper_chwidth=1
he_oper_centr_freq_seg0_idx=${freq_seg0_idx}
he_default_pe_duration=4
he_rts_threshold=1023
he_mu_edca_qos_info_param_count=0
he_mu_edca_qos_info_q_ack=0
he_mu_edca_qos_info_queue_request=0
he_mu_edca_qos_info_txop_request=0
he_mu_edca_ac_be_aifsn=8
he_mu_edca_ac_be_aci=0
he_mu_edca_ac_be_ecwmin=9
he_mu_edca_ac_be_ecwmax=10
he_mu_edca_ac_be_timer=255
he_mu_edca_ac_bk_aifsn=15
he_mu_edca_ac_bk_aci=1
he_mu_edca_ac_bk_ecwmin=9
he_mu_edca_ac_bk_ecwmax=10
he_mu_edca_ac_bk_timer=255
he_mu_edca_ac_vi_ecwmin=5
he_mu_edca_ac_vi_ecwmax=7
he_mu_edca_ac_vi_aifsn=5
he_mu_edca_ac_vi_aci=2
he_mu_edca_ac_vi_timer=255
he_mu_edca_ac_vo_aifsn=5
he_mu_edca_ac_vo_aci=3
he_mu_edca_ac_vo_ecwmin=5
he_mu_edca_ac_vo_ecwmax=7
he_mu_edca_ac_vo_timer=255
he_su_beamformer=1
he_su_beamformee=1
he_mu_beamformer=1
EOF
	elif [[ ${netcard_model} =~ "MT7921" ]]; then
		cat >> ${hostapd_cfgpath} << EOF
ht_capab=[RXLDPC][HT40+][SHORT-GI-20][SHORT-GI-40][TX-STBC][RX-STBC1][MAX-AMSDU-7935]
vht_capab=[MAX-MPDU-7991][RXLDPC][SHORT-GI-80][TX-STBC][SU-BEAMFORMEE][MU-BEAMFORMEE][RX-ANTENNA-PATTERN][TX-ANTENNA-PATTERN]

ieee80211ax=1
he_oper_chwidth=1
he_oper_centr_freq_seg0_idx=${freq_seg0_idx}
he_default_pe_duration=4
he_rts_threshold=1023
he_mu_edca_qos_info_param_count=0
he_mu_edca_qos_info_q_ack=0
he_mu_edca_qos_info_queue_request=0
he_mu_edca_qos_info_txop_request=0
he_mu_edca_ac_be_aifsn=8
he_mu_edca_ac_be_aci=0
he_mu_edca_ac_be_ecwmin=9
he_mu_edca_ac_be_ecwmax=10
he_mu_edca_ac_be_timer=255
he_mu_edca_ac_bk_aifsn=15
he_mu_edca_ac_bk_aci=1
he_mu_edca_ac_bk_ecwmin=9
he_mu_edca_ac_bk_ecwmax=10
he_mu_edca_ac_bk_timer=255
he_mu_edca_ac_vi_ecwmin=5
he_mu_edca_ac_vi_ecwmax=7
he_mu_edca_ac_vi_aifsn=5
he_mu_edca_ac_vi_aci=2
he_mu_edca_ac_vi_timer=255
he_mu_edca_ac_vo_aifsn=5
he_mu_edca_ac_vo_aci=3
he_mu_edca_ac_vo_ecwmin=5
he_mu_edca_ac_vo_ecwmax=7
he_mu_edca_ac_vo_timer=255
he_su_beamformer=1
he_su_beamformee=1
he_mu_beamformer=1
EOF
	elif [[ ${netcard_model} =~ "Qualcomm Device 1104" ]]; then
		# 该网卡支持 160MHz 与 80MHz
		cat >> ${hostapd_cfgpath} << EOF
# 不同厂商的卡 支持的参数不同。
# HT capabilities
# ht_capab=[HT40+][HT20-][HT40-][SHORT-GI-20][SHORT-GI-40][RX-STBC1][DSSS_CCK-40]

# VHT capabilities
# vht_capab=[MAX-MPDU-11454][SHORT-GI-80][SHORT-GI-160][RX-STBC-1][TX-STBC][SU-BF][SU-BFEE][MU-BF][RX-ANTPATTERN][TX-ANTPATTERN][VHT160]

ht_capab=[LDPC][HT40+][SHORT-GI-20][SHORT-GI-40][TX-STBC][RX-STBC1][DSSS_CCK-40]
require_ht=1

vht_capab=[MAX-MPDU-11454][RXLDPC][SHORT-GI-80][TX-STBC-2BY1][RX-STBC-1][MAX-A-MPDU-LEN-EXP7]
require_vht=1

ieee80211ax=1
he_oper_chwidth=1
he_oper_centr_freq_seg0_idx=${freq_seg0_idx}


EOF
	elif [[ ${netcard_model} =~ "Qualcomm Atheros QCNFA765" ]]; then
		# 支持 160MHz
		cat >> ${hostapd_cfgpath} << EOF
# HT capabilities
ht_capab=[HT40+][HT20-][HT40-][SHORT-GI-20][SHORT-GI-40][RX-STBC1][DSSS_CCK-40][MAX-AMSDU-7935]

# VHT capabilities
vht_capab=[MAX-MPDU-11454][SHORT-GI-80][SHORT-GI-160][RX-STBC-1][TX-STBC][SU-BF][SU-BFEE][MU-BF][RX-ANTPATTERN][TX-ANTPATTERN][VHT160]

ieee80211ax=1
he_oper_chwidth=1
he_oper_centr_freq_seg0_idx=${freq_seg0_idx}

he_su_beamformer=1
he_mu_beamformer=1
he_default_pe_duration=4
he_rts_threshold=1023
he_mu_edca_qos_info_param_count=0
he_mu_edca_qos_info_q_ack=0
he_mu_edca_qos_info_queue_request=0
he_mu_edca_qos_info_txop_request=0
he_mu_edca_ac_be_aifsn=8
he_mu_edca_ac_be_aci=0
he_mu_edca_ac_be_ecwmin=9
he_mu_edca_ac_be_ecwmax=10
he_mu_edca_ac_be_timer=255
he_mu_edca_ac_bk_aifsn=15
he_mu_edca_ac_bk_aci=1
he_mu_edca_ac_bk_ecwmin=9
he_mu_edca_ac_bk_ecwmax=10
he_mu_edca_ac_bk_timer=255
he_mu_edca_ac_vi_ecwmin=5
he_mu_edca_ac_vi_ecwmax=7
he_mu_edca_ac_vi_aifsn=5
he_mu_edca_ac_vi_aci=2
he_mu_edca_ac_vi_timer=255
he_mu_edca_ac_vo_aifsn=5
he_mu_edca_ac_vo_aci=3
he_mu_edca_ac_vo_ecwmin=5
he_mu_edca_ac_vo_ecwmax=7
he_mu_edca_ac_vo_timer=255
EOF
	elif [[ ${netcard_model} =~ "RTL8821AE" ]]; then
		cat >> ${hostapd_cfgpath} << EOF
ht_capab=[HT40+][SHORT-GI-20][SHORT-GI-40][MAX-AMSDU-7935][DSSS_CCK-40]
vht_capab=[SHORT-GI-80][MAX-MPDU-11454][TX-STBC][SU-BEAMFORMER][SU-BEAMFORMEE][HTC-VHT][RX-ANTENNA-PATTERN][TX-ANTENNA-PATTERN]
EOF
	elif [[ ${netcard_model} =~ "BCM43602" ]]; then # don't work
		cat >> ${hostapd_cfgpath} << EOF
ht_capab=[HT40+][SHORT-GI-20][SHORT-GI-40]
vht_capab=[SHORT-GI-80][SU-BEAMFORMER][SU-BEAMFORMEE]
EOF
	elif [[ ${netcard_model} =~ "AR958x" ]]; then # don't work
		cat >> ${hostapd_cfgpath} << EOF
ht_capab=[HT40+][SHORT-GI-20][SHORT-GI-40]
EOF
	elif [[ ${netcard_model} =~ "AR93xx" ]]; then
		cat >> ${hostapd_cfgpath} << EOF
ht_capab=[RXLDPC][HT40+][SHORT-GI-20][SHORT-GI-40][TX-STBC][RX-STBC1][DSSS_CCK-40]
EOF
	else
		cat >> ${hostapd_cfgpath} << EOF
ht_capab=[HT40+][SHORT-GI-20][SHORT-GI-40]
vht_capab=[RXLDPC][SHORT-GI-80]
EOF
	fi

	hostapd -B ${hostapd_cfgpath} >> /etc/router/run/wifi.log
	start_times=1
	for i in {1..2}
	do
		no_ir=`iw list |egrep "(5180|5200|5220|5240|5745|5765|5785|5805) MHz" | grep "no IR" | wc -l`
		echo `date +"%F %T"`" i = $i, start_times = $start_times, state = ${state},no_ir = ${no_ir}" >> /etc/router/run/wifi.log
		proc=`ps -ef | grep ${hostapd_cfgpath} | grep -v grep | awk '{print $2}'`
		if [[ -z ${proc} ]]; then
			let start_times++
			sleep 3
			no_ir=`iw list |egrep "(5180|5200|5220|5240|5745|5765|5785|5805) MHz" | grep "no IR" | wc -l`
			if [[ ${chwidth} == 1 && ${no_ir} == 0 ]]; then
				sed -i 's/vht_oper_chwidth=.*/vht_oper_chwidth=1/g' ${hostapd_cfgpath}
			else
				# 80MHz -> 20 or 40 MHz
				sed -i 's/vht_oper_chwidth=.*/vht_oper_chwidth=0/g' ${hostapd_cfgpath}
			fi
			hostapd -B ${hostapd_cfgpath} >> /etc/router/run/wifi.log
		else
			ap_status=`hostapd_cli -i ${ifname} status | grep state | awk -F= '{print $2}'`
			if [[ ${ap_status} == "ENABLED" ]]; then
				end=`date +"%s"`
				echo `date +"%F %T"`"-------------end------------" >> /etc/router/run/wifi.log
				echo "$[end-start]s" >> /etc/router/run/wifi.log
				echo "${ifname} enabled success!"
				return 0
			fi
		fi
		sleep 3
	done
	echo "${ifname} enabled failed!" >&2
	return 1
}

check_wireless_channel() {
	local netdev=$1
	wireless_dir="/sys/class/net/"${netdev}"/wireless"
	if [[ -d ${wireless_dir} ]]; then
		phy_name=`cat /sys/class/net/${netdev}/phy80211/name`
		support_5g=`iw ${phy_name} channels | grep "Band 2:"`
		if [[ -n ${support_5g} ]]; then
			start_wireless_5g ${netdev}
		fi
	fi
}

# Wi-Fi 重试
# retry () {
# 	local hostapd_cfgpath=$1
	
# 	vht_oper_centr_freq_seg0_idx=${freq_seg0_idx}
# 	channel=${channel}
# 	# change 
# }

scan_wireless_dev() {
	netdevs=`ls /sys/class/net/ | grep -v "$(ls /sys/devices/virtual/net/)"`
	for netdev in ${netdevs[@]}
	do
		check_wireless_channel ${netdev}
	done
}

reinstall_driver(){
	local ifname=$1
	local bus_id=`ls -l /sys/class/net/${ifname}/device | awk -F'0000:' '{print $2}'`
	local netcard_model=`lspci -s ${bus_id} | awk -F': ' '{print $2}'`

	# QCA9984
	# rmmod ath10k_core
	# rmmod ath

	# RTL8821AE
	
	if [[ ${netcard_model} =~ "QCA9984" ]]; then
			modprobe -r ath10k_pci
			# QCA9984
			echo "QCA9984"
			modprobe ath10k_pci
	elif [[ ${netcard_model} =~ "RTL8821AE" ]]; then
		# RTL8821AE
		echo "RTL8821AE"
		
		modprobe -r rtl8821ae
		# rmmod rtl8821ae 
		# rmmod rtl_pci 
		# rmmod btcoexist
		# rmmod rtlwifi 

		# rmmod mac80211
		# rmmod cfg80211

		# modprobe rtl_pci 
		# modprobe rtlwifi 
		# modprobe rtl8821ae
		# modprobe btcoexist
		
		# modprobe cfg80211
		# modprobe lib80211
		# modprobe mac80211
		# modprobe libarc4
		modprobe rtl8821ae
	elif [[ ${netcard_model} =~ "MT7915E" ]]; then
		modprobe -r mt7915e
		modprobe mt7915e
	fi

	
}

cfgfile_wan_method=/etc/router/config/wan_method.json
setup=`cat ${cfgfile_wan_method} | jq -r .setup`

if [[ ${setup} == 7 ]]; then
	cfgfile_wan=/etc/router/config/wan_wireless_repeater.json
	default_channel=`cat ${cfgfile_wan} | jq -r .channel`
	if [[ ${default_channel} -lt 36 ]]; then
		exit 0
	fi
	netdev=wlan0-ap
	start_wireless_5g ${netdev}
else
	scan_wireless_dev
fi
