#!/bin/bash

# cfgfile=/etc/router/config/storage_create_pool.json
cfgfile=$1

raid_level=`cat ${cfgfile} | jq -r .raid_level`
disk_num=`cat ${cfgfile} | jq -r .drive_num`
disk_list=
pool_name=

# convert raid level
if [[ ${raid_level} == raid0 ]]; then
	disk_list=""
elif [[ ${raid_level} == raid1 ]]; then
	disk_list="mirror"
elif [[ ${raid_level} == raid5 ]]; then
	disk_list="raidz1"
elif [[ ${raid_level} == raid6 ]]; then
	disk_list="raidz2"
elif [[ ${raid_level} == raid10 ]]; then
	disk_list=""
else
	echo "Not support raid type!" >&2
	exit 1
fi

# assemble a list of disks
for i in $(seq 0 $[$disk_num-1])
do
	disk=`cat ${cfgfile} | jq -r .drives[$i]`
	mount_disk=`df | awk '{print $1}' | grep "^${disk}"`
	if [[ -n ${mount_disk} ]]; then
		umount ${mount_disk}
	fi
	# TODO 有些时候单盘有两个id 
	# 必须要选到主盘的id
	disk_id=`udevadm info ${disk} -q symlink -r | sed 's/ /\n/g' | grep by-id | sort | head -n 1`
	if [[ ${raid_level} == raid10 && $[$i%2] == 0 ]]; then
		disk_list=${disk_list}" mirror "${disk_id}
	else
		disk_list=${disk_list}" "${disk_id}
	fi
done

# get an unused pool name
for i in $(seq 0 99)
do
	datadir="/data"$i
	mount_points=`df | awk '{print $6}' | grep "^${datadir}"`
	if [[ -z ${mount_points} ]]; then
		mkdir -p ${datadir}
		pool_name="data"$i
		break
	fi
done
if [[ -z ${pool_name} ]]; then
	echo "Could not find unused pool name..." >&2
	exit 1
fi

systemctl daemon-reload
# create new pool
zpool create -f ${pool_name} ${disk_list}
sleep 0.5
chmod 777 /${pool_name}

# check mount resule
mount_dev=`df | awk '{print $1}' | grep "^${pool_name}"`
if [[ -z ${mount_dev} ]]; then
	echo "${pool_name} is not mounted, it will be destroyed..." >&2
	zpool destroy -f ${pool_name}
	exit 1
fi
