#!/bin/bash

# cfgfile=/etc/router/config/storage_create_volume.json
cfgfile=$1
datadir=$2

get_unused_datadir() {
	if [[ ${datadir} != "" ]]; then
			return
	fi

	for i in $(seq 0 99)
	do
			datadir="/data"$i
			mount_points=`df | awk '{print $6}' | grep "^${datadir}"`
			if [[ -z ${mount_points} ]]; then
					mkdir -p ${datadir}
					chmod 777 ${datadir}
					return
			fi
	done
	echo "Could not find unused datadir..." >&2
	exit 1
}

dev_formatting() {
	local dev=$1
	local fstype=$2

	echo "start to format the dev"
	# formatting partition
	if [[ ${fstype} == xfs ]]; then
			# exfat can clear superblock info
			mkfs.exfat ${dev}
			mkfs.xfs -f ${dev}
	elif [[ ${fstype} == exfat ]]; then
			mkfs.exfat ${dev}
	else
			echo "disk fstype unknown..." >&2
			exit 1
	fi
	sleep 0.5
}

dev_mount() {
	echo "start to mount the dev"
	LOCK_FILE=/etc/router/run/create_volume.lock # lock file， 上锁文件不同则不会互斥
	exec 99> "${LOCK_FILE}" # 表示创建文件描述符99，指向锁文件，此处值不同也会互斥，
	flock -e 99

	local dev=$1
	local fstype=$2

	# mount partition
	get_unused_datadir
	echo "datadir is ${datadir}"
	local uuid=`lsblk -no uuid ${dev} | head -1`
	#local fstype=`lsblk -no fstype ${dev} | head -1`
	if [[ -z ${uuid} ]]; then
			echo "${dev} get uuid failed..." >&2
			flock -u 99
			exit 1
	fi
	sed -i "\#  ${datadir}  #d" /etc/fstab
	if [[ ${fstype} == xfs ]]; then
			echo "UUID=${uuid}  ${datadir}  ${fstype}  defaults  0  0 " >>/etc/fstab
	else
			echo "UUID=${uuid}  ${datadir}  ${fstype}  defaults,umask=0000  0  0 " >>/etc/fstab
	fi
	sleep 0.5
	systemctl daemon-reload
	mount -a
	chmod 777 ${datadir}
	flock -u 99
	echo "finish mounting the dev"
}

disk_single_legacy() {
	disk=`cat ${cfgfile} | jq -r .path`
	fstype=`cat ${cfgfile} | jq -r .fstype`
	diskdev=${disk}1
	if [[ ${disk} =~ "nvme" ]]; then
			diskdev=${disk}p1
	fi

	# function 1:
	# bug: can not recognized on windows
	# https://askubuntu.com/questions/706608/exfat-external-drive-not-recognized-on-windows
	# parted disk
	# parted -s ${disk} mklabel gpt mkpart primary 0% 100%

	echo "start creating gpt partition on dev"
	# function 2:
	parted -s ${disk} mklabel gpt
	parted -a optimal ${disk} mkpart primary '0%' '100%'
	parted ${disk} set 1 msftdata on

	sleep 0.5

	partprobe
	# finish creating gpt partition on dev. after that we need sleep 1 second at least to wait system loading
	sleep 1
	dev_formatting ${diskdev} ${fstype}
	dev_mount ${diskdev} ${fstype}

	# check mount result
	mount_dev=`df | awk '{print $1}' | grep "^${diskdev}"`
	if [[ -z ${mount_dev} ]]; then
		echo "${diskdev} is not mounted, it will be deleted..." >&2
		sed -i "\#  ${datadir}  #d" /etc/fstab
		exit 1
	fi

	exit 0
} 

disk_single_zfs() {
	disk=`cat ${cfgfile} | jq -r .path`
	disk_id=`udevadm info ${disk} -q symlink -r | sed 's/ /\n/g' | grep by-id`
	pool_name=

	# get an unused pool name
	for i in $(seq 0 99)
	do
		datadir="/data"$i
		mount_points=`df | awk '{print $6}' | grep "^${datadir}"`
		if [[ -z ${mount_points} ]]; then
			mkdir -p ${datadir}
			pool_name="data"$i
			break
		fi
	done
	if [[ -z ${pool_name} ]]; then
		echo "Could not find unused pool name..." >&2
		exit 1
	fi

	zpool create -f ${pool_name} ${disk_id}

	# check mount resule
	mount_dev=`df | awk '{print $1}' | grep "^${pool_name}"`
	if [[ -z ${mount_dev} ]]; then
		echo "${pool_name} is not mounted, it will be destroyed..." >&2
		zpool destroy -f ${pool_name}
		exit 1
	fi

	exit 0
}

disk_single() {
	fstype=`cat ${cfgfile} | jq -r .fstype`

	if [[ ${fstype} == zfs ]]; then
		disk_single_zfs
	else
		disk_single_legacy
	fi
}

disk_raid() {
	exit 0
}

type=`cat ${cfgfile} | jq -r .type`
case "${type}" in
  0)
    disk_single
    ;;
  1)
    disk_raid
    ;;
  *)
    exit 1
esac